//package org.enhydra.jdbc.standard;
import java.io.PrintWriter;
import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.sql.*;
import org.enhydra.jdbc.standard.StandardDataSource;
import org.enhydra.jdbc.standard.StandardXADataSource;
import org.enhydra.jdbc.standard.StandardPooledConnection;
import org.enhydra.jdbc.standard.StandardConnectionPoolDataSource;

public class J2poolTest extends StandardDataSource implements ConnectionEventListener {

	Vector						pool;		// the pool of connections
	ConnectionPoolDataSource	ds;			// the source for pooled connections

	static String dataName = "jdbc/pool";
	static String dataSourceClass = "org.enhydra.jdbc.standard.StandardConnectionPoolDataSource";

	public J2poolTest () {
		super();
		pool = new Vector (10,5);							// create the connection pool
		try {
			Context initCtx = J2stextTest.getInitialContext();

			try {
				// Try to get the data source for pooled connections
				ds = (ConnectionPoolDataSource)initCtx.lookup (dataName);
			} catch (Exception e) {
				// probably not bound yet
				StandardConnectionPoolDataSource jds = 
					(StandardConnectionPoolDataSource)J2stextTest.bindDs (
						initCtx, dataName, dataSourceClass);
				ds = jds;
			} // try-catch
		} catch (Exception e) {
			e.printStackTrace();
		} // try-catch

	} // method J2poolTest

	public void setLogWriter (PrintWriter out) {
		super.setLogWriter (out);							// set our own log
		try {
			ds.setLogWriter (out);							// and set the pooled connection data source log
		} catch (SQLException e) {
			e.printStackTrace();
		} // try-catch
	} // method setLogWriter

	public Connection getConnection () throws SQLException {
		return getConnection (null, null);
	} // method getConnection

	PooledConnection getPooledConnection (String user, String passwd, boolean xa) throws SQLException {
		PooledConnection con=null;

		synchronized (pool) {
			if (pool.size()==0) {
				// no pooled connections left
				if (xa) {
					StandardXADataSource xads = (StandardXADataSource)ds;
					con = xads.getXAConnection (user, passwd);
				} else {
					con = ds.getPooledConnection (user, passwd);
				} // if-else
				con.addConnectionEventListener (this);
			} else {
				// get connection from pool
				Object obj = pool.lastElement();
				pool.removeElement (obj);
				con = (PooledConnection)obj;
			} // if-else
		} // synchronized (pool) 
		return con;
	} // method getPooledConnection

	public Connection getConnection (String user, String passwd) throws SQLException {
		PooledConnection con=getPooledConnection(user, passwd,false);
		Connection ret = con.getConnection();
		return ret;
	} // method getConnection

	public void connectionClosed(ConnectionEvent event) {
		Object obj = event.getSource();
		pool.addElement (obj);
	}


	/**
	 * Used to shut down this test.
	 */
	void close () throws SQLException {
		for (int i=0; i<pool.size(); i++) {
			Object o = pool.elementAt(i);
			StandardPooledConnection con = (StandardPooledConnection)o;
			con.close();
		} // for
	} // method close

	public void connectionErrorOccurred(ConnectionEvent event) {
	}

	/**
	 * The factory interface.
	 */
	public Object getObjectInstance(Object refObj, Name name, Context nameCtx, Hashtable env) throws Exception {
		Reference ref = (Reference)refObj;
		if (ref.getClassName().equals(getClass().getName())) {
			J2poolTest dataSource = new J2poolTest ();
			dataSource.setDriverName((String)ref.get("driverName").getContent());
			dataSource.setUrl((String)ref.get("url").getContent());
			return dataSource;
		} else {
			return null;
		}
	}

} // class J2poolTest