//package org.enhydra.jdbc.standard;
import java.io.*;
import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.sql.*;
import org.enhydra.jdbc.standard.StandardDataSource;

/**
 * A little test program to try out JDBCDataSource.
 *
 * It uses the file system JNDI provider to save objects.
 */
public class J2stextTest {

	static String dataName	= "jdbc/sample";
	static String poolName	= "jdbc/poolsample";
	static String xaName	= "jdbc/xasample";
	static String idbxaName = "jdbc/idbsample";

	static String[] sql = {
		"DROP TABLE tmp",
		"CREATE TABLE tmp (int1 int, char1 char(10))",
		"INSERT INTO tmp VALUES (?, ?)",
		"SELECT * FROM TMP",
		"SHUTDOWN"
	};

	static void executeSql (Connection con, int start, int end) throws SQLException {
		Statement s = con.createStatement();
		for (int i=start; i<=end; i++) {
			String sqls = sql[i];
			if (sqls.indexOf("?")==-1) {
				s.execute(sqls);
			} else {
				PreparedStatement p = con.prepareStatement (sqls);
				p.setInt (1,i);
				p.setString (2,"hello");
				p.execute();
			} // if-else
		} // for
	} // method executeSql

	static StandardDataSource bindDs (Context context, String name, String className) {
		StandardDataSource jds = null;
		try {
			// We create a simple JDBC Data source that needs only a
			// JDBC driver and a JDBC URL in order to connect.
			jds = (StandardDataSource)(Class.forName(className).newInstance());
			jds.setUrl ("jdbc:idb:C:/idb/Testing/Databases/sample/sample.prp");
			jds.setDriverName ("org.enhydra.instantdb.jdbc.idbDriver");

			// Bind the Data source in JNDI
			context.bind (name, jds);
		} catch (Exception e) {
			e.printStackTrace();
		} // try-catch
		return jds;
	} // method bindDs

	static Context getInitialContext () {
		Context initCtx = null;
		try {
			// Set up the file system JNDI provider to create the
			// initial context.
			Hashtable env = new Hashtable(11);
			env.put(Context.INITIAL_CONTEXT_FACTORY,
					"com.sun.jndi.fscontext.RefFSContextFactory");
			env.put(Context.PROVIDER_URL,
					"file:///c:/idb/testing/jndi");
			initCtx = new InitialContext(env);
		} catch (Exception e) {
			e.printStackTrace();
		} // try-catch
		return initCtx;
	} // method getInitialContext

	public static void main (String[] args) {
		try {
			Context initCtx = getInitialContext();
//			PrintWriter log = new PrintWriter(new FileOutputStream ("log.txt"));
			PrintWriter log = new PrintWriter(System.out);

			DataSource ds = null;
			try {
				// Try to find the database
				ds = (DataSource)initCtx.lookup (dataName);
			} catch (Exception e) {
				// probably not bound yet
				StandardDataSource jds = bindDs (initCtx, dataName, 
					"org.enhydra.jdbc.standard.StandardDataSource");
				ds = jds;
			} // try-catch

			// Obtain a database connection
			ds.setLogWriter (log);
			Connection con = ds.getConnection ();
			con.close();

			// Do the whole thing again, but this time use a pooled connection.
			try {
				// Try to find the database
				ds = (DataSource)initCtx.lookup (poolName);
			} catch (Exception e) {
				// probably not bound yet
				StandardDataSource jds = bindDs (initCtx, poolName, "J2poolTest");
				ds = jds;
			} // try-catch

			// Obtain a database connection
			ds.setLogWriter (log);
			con = ds.getConnection ();
			executeSql (con, 0, 3);
			con.close();

			con = ds.getConnection ();
			executeSql (con, 2, 3);
			con.close();

			con = ds.getConnection ();
			executeSql (con, 2, 3);
			con.close();

			J2poolTest tst = (J2poolTest)ds;
			tst.close();

			// Do the whole thing again, but this time use an XA connection.
			try {
				// Try to find the database
				ds = (DataSource)initCtx.lookup (xaName);
			} catch (Exception e) {
				// probably not bound yet
				StandardDataSource jds = bindDs (initCtx, xaName, "J2xaTest");
				ds = jds;
			} // try-catch

			// Obtain a database connection
			ds.setLogWriter (log);
			con = ds.getConnection ();
			executeSql (con, 0, 3);
			con.close();

			con = ds.getConnection ();
			executeSql (con, 2, 3);
			// Cause a timeout on the transaction.
			Thread.sleep(30000);
			con.close();

			con = ds.getConnection ();
			executeSql (con, 2, 3);
			con.close();

			tst = (J2poolTest)ds;
			tst.close();

			// Do the whole thing again, but this time use an InstantDB XA connection.
			try {
				// Try to find the database
				ds = (DataSource)initCtx.lookup (idbxaName);
			} catch (Exception e) {
				// probably not bound yet
				StandardDataSource jds = bindDs (initCtx, idbxaName, "J2idbxaTest");
				ds = jds;
			} // try-catch

			// Obtain a database connection
			ds.setLogWriter (log);
			con = ds.getConnection ();
			executeSql (con, 0, 3);
			con.close();

			// just do a read
			ds.setLogWriter (log);
			con = ds.getConnection ();
			executeSql (con, 3, 3);
			con.close();

			// reuse the same connection
			con = ds.getConnection ();
			executeSql (con, 2, 3);

			// before closing 1st connection, get a second one
			Connection con2 = ds.getConnection ();
			executeSql (con2, 2, 3);
			try {
				// we expect this to fail
				con.commit ();
			} catch (Exception e) {
				e.printStackTrace();
			} // try-catch
			con.close();
			con2.close();

			con = ds.getConnection ();
			executeSql (con, 3, 3);
			con.close();

			tst = (J2poolTest)ds;
			tst.close();

		} catch (Exception e) {
			e.printStackTrace();
		} // try-catch
	} // method main

} // class J2stextTest 
