/*
*   $Id: CCMerchantTest.c,v 1.14 1998/03/11 20:34:37 vchen Exp $
*
*   System     :   Merchant Connection Kit
*
*   Author     :   CYCH Development staff
*   Description:   Module of customizable functions shared by MCK CGI scripts
*   Notes      :   Each merchant should have a copy of this which is customized
*                  to integrate with the merchant storefront
*
*                         COPYRIGHT NOTICE
*
*   The contents of this file is protected under the United States
*   copyright laws as an unpublished work, and is confidential and
*   proprietary to CyberCash, Inc.  Its use or disclosure in whole or in
*   part without the expressed written permission of CyberCash, Inc. is
*   prohibited.
*
*   (c) Copyright 1993-97 by CyberCash, Inc.  All rights reserved.
*
*/                                                                       

/* Warning!  pointer arithmetic here that may not work with */
/*           multi-byte character sets!                     */


#ifndef WIN32
#include <unistd.h>
#else
 typedef long pid_t;
 #include <process.h>
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#ifndef BSDI
#   include <malloc.h>
#endif

#include "CCErrno.h"
#include "CCNVlist.h"
#include "CCMckLib.h"
#include "CCString.h"
#include "CCBlock.h"

#include "CCConfig.h"
#include "CCMerchantTest.h"


   /* MAXTMPLINE should be bigger than biggest line you expect in a template. */
#define MAXTMPLINE 1024

	/* MAXLOGLINE should be bigger than anything you expect to see in the */
	/* notification log                                                   */
#define MAXLOGLINE 4000

extern THREAD char msgBuf[MAXTMPLINE];

#ifdef WIN32
	/* IIS on NT can't handle relative files */
const THREAD char *configFile = "/infosystem/www/htdocs/membres/gibello/CyberCash/test-mck/mck-cgi/conf\\merchant_conf";
#else
	/* an absolute path can be used by ALL scripts, not just CGIs */
const THREAD char *configFile = "/infosystem/www/htdocs/membres/gibello/CyberCash/test-mck/mck-cgi/conf/merchant_conf";   /* we expect it here... */
#endif

/*============================================================================
 * Merchant constants
 *===========================================================================*/

/* This is here for total melt-down of templating. */
/* Otherwise, templates will be expanded for messaging. */
/* template routines are in libCCMck */

void TemplateFailure(char* message) 
{
	printf("Content-type: text/html\r\n\r\n");
	
	printf("<html>\r\n");
	printf("\r\n");
	printf("<head>\r\n");
	printf("<title>Temporary Difficulties</title>\r\n");
	printf("</head>\r\n");
	printf("\r\n");
	printf("<body>\r\n");
	printf("<h1>Temporary Difficulties</h1>\r\n");
	printf("\r\n");
	printf("<p>%s is experiencing \r\n", "CyberCash MCK Test Merchant");
	printf("temporary difficulties.</p>\r\n");
	printf("\r\n");

		/* message prints here! */
	printf("%s\r\n", message);

	printf("\r\n");
	printf("<p>Please contact %s\r\n", "CyberCash MCK Test Merchant");
	printf("customer service staff at %s, \r\n", "merchant-support@cybercash.com");
	printf("or try again later.</p>\r\n");
	printf("\r\n");
	printf("<p>Sorry for the inconvenience.</p>\r\n");
	printf("\r\n");
	printf("</body>\r\n");
	printf("</html>\r\n");

}

/**
 * EXPERSHOP
 * Remove SaveOrderInfo
 * Remove GetOrderInfo
 */
/*
int SaveOrderInfo (
	char* orderId, char* price, char* orderValidTill, char* payload, 
	char* orderDescr,char* customerId) 

int GetOrderInfo (char* orderId, OrderRecPtr result) 
*/

/*
******************************
* sub CheckOrderId
*
* When the order was generated, the OrderId was saved.
* Now, check to see if the OrderId on the notification is an OrderId 
* that you recognize.
*
*  See comments in SaveOrderInfo to see how the test script implements
*  the Order Log.
*  
* I'm using a flat file, but this will not scale to a large, complex
*  storefront.  You will need to replace this function to work with
*  the kind of data you store and the method you use to store that data.
*
****************************
*/

int CheckOrderId (char* orderId)
{
/* EXPERSHOP... Temporarily always return 1 (= true) */
  return 1;
}


/*
***********************************
*  LogNotification
*
*  Write the POP to the notification log
*  If you want to change the format or what is saved,
*  make the  changes here
*
* We are saving the POP string as it arrived...
*
*  return error code on failure
*  return CC_SUCCESS on success
*
****************************************
*/

/*
 * EXPERSHOP  This routine is left unchanged for backup purposes
 * You may clear its content if you don't need a log
 */
int LogNotification(char* POPstr)
{

	FILE* log_fd = NULL;
	char *logName = NULL;

	if (NULL == Config) {
		CCLogError("%s: %s\n", progname,
			CCGetErrorMessage(CC_ENULL_CONFIG));
		return CC_ENULL_CONFIG;
	}

	logName = Config->notificationLog;

	log_fd = fopen(logName, "a");

	if ( NULL == log_fd ) {
		CCLogError("%s: %s (%s)\n", progname, 
			CCGetErrorMessage(CC_ENO_NOTIFLOG), logName);
		return CC_ENO_NOTIFLOG;  /* no notification log */
	}

	fprintf(log_fd,"%s\n", POPstr);
	fclose(log_fd);

	return CC_SUCCESS;
}

/*
***********************************
*  ExtractNotification
*
*  Input:  An order ID	
*  Return:   An error code   (0 = OK, > 0  errno: file problem,
*                           -1 if record not found)
*  Output parameters:
*          An error message (if applicable)
*          An CCNVlistPtr vector containing the fields of the POP record
*  
*  If you change the Notification Log format or what is saved in that log,
*  You will have to make matching changes here...
*
*  Note: The POP string is stored in URL encoded format,
*	exactly as it arrived in the notification/fulfillment cgi...
*
****************************************
*/

/*
 * EXPERSHOP  This routine is left unchanged for backup purposes
 * It is not used in the CGI scripts
 * You can use it for backup purposes in your own programs
 * (if so, keep LogNotification() unchanged).
 */
CCNVlistPtr ExtractNotification(char* orderId, int* errcode, char* message)
{
	FILE* log_fd;
	char *logName = NULL;
	char buf[MAXLOGLINE];
	char* cp = NULL;
	size_t ix;
	char encodedOid[MAXTMPLINE];
	char oid[MAXTMPLINE];

	CCNVlistPtr POP;

	if (NULL == Config) {
		*errcode = CC_ENULL_CONFIG;
		sprintf(message, "%s: %s", progname, CCGetErrorMessage(*errcode));
		return NULL;  /* no config, no work */
	}

	logName = Config->notificationLog;

	log_fd = fopen(logName, "r");

	if ( NULL == log_fd ) {

		*errcode = CC_ENO_NOTIFLOG;

		sprintf(message, "%s: %s (%s) [%d]\n", progname, 
			CCGetErrorMessage(*errcode), logName, errno);

		return NULL; /* can't open the file */
	}

    /* sigh! linear search... */
    while (!feof(log_fd)) {
        fgets(buf, MAXLOGLINE, log_fd);
        if ((cp = strstr(buf, "pop.order-id=")) != NULL) {
			/* cp points to pop.order-id */	

			cp += strlen("pop.order-id="); 

			/* cp is start of order-id */
			ix = strcspn(cp, "&");     /* find end of it */

			strncpy(encodedOid, cp, ix);
			encodedOid[ix] = '\0';

			/* urldecode this... */
			URLDecode(encodedOid, oid, MAXTMPLINE);

			/* NOW you can compare this and the order-id */
			if (strcmp(orderId, oid) == 0) {
				/* MATCH! */
				POP = CCUrlDecode(buf);  /* unwrap it */
				message[0] = '\0';
				fclose(log_fd);
				*errcode = CC_SUCCESS;
				return POP;

			}
			/* else, keep looking */
        }
    }

	fclose(log_fd);

	*errcode = -1;
	return NULL;  /* not found */
}

